@@ -8,6 +8,7 @@ const R_CallMethodDef callMethods[] = {
   {"Decode_Junction", (DL_FUNC) &Decode_Junction, 1},
   {"Decode_Sample", (DL_FUNC) &Decode_Sample, 2},
   {"Decode_LBP", (DL_FUNC) &Decode_LBP, 4},
+  {"Decode_RBP", (DL_FUNC) &Decode_RBP, 4},
   {"Decode_TRBP", (DL_FUNC) &Decode_TRBP, 4},
   {"Decode_Greedy", (DL_FUNC) &Decode_Greedy, 3},
   {"Decode_ICM", (DL_FUNC) &Decode_ICM, 3},
@@ -18,6 +19,7 @@ const R_CallMethodDef callMethods[] = {
   {"Infer_Junction", (DL_FUNC) &Infer_Junction, 1},
   {"Infer_Sample", (DL_FUNC) &Infer_Sample, 2},
   {"Infer_LBP", (DL_FUNC) &Infer_LBP, 4},
+  {"Infer_RBP", (DL_FUNC) &Infer_RBP, 4},
   {"Infer_TRBP", (DL_FUNC) &Infer_TRBP, 4},
   {"Sample_Exact", (DL_FUNC) &Sample_Exact, 2},
   {"Sample_Chain", (DL_FUNC) &Sample_Chain, 2},
