@@ -4,8 +4,6 @@ SEXP MRF_Update(SEXP _crf)
 {
 	CRF crf(_crf);
 	crf.Update_Pot();
-	crf.Normalize_NodePot();
-	crf.Normalize_EdgePot();
 	return (_crf);
 }
 
@@ -13,8 +11,6 @@ SEXP CRF_Update(SEXP _crf, SEXP _nodeFea, SEXP _edgeFea, SEXP _nodeExt, SEXP _ed
 {
 	CRF crf(_crf);
 	crf.Update_Pot(_nodeFea, _edgeFea, _nodeExt, _edgeExt);
-	crf.Normalize_NodePot();
-	crf.Normalize_EdgePot();
 	return (_crf);
 }
 
@@ -145,11 +141,7 @@ void CRF::Update_Pot(SEXP _nodeFea, SEXP _edgeFea, SEXP _nodeExt, SEXP _edgeExt)
 		}
 	}
 
-	for (int i = 0; i < nNodes * maxState; i++)
-		nodePot[i] = exp(nodePot[i]);
-	for (int i = 0; i < nEdges; i++)
-		for (int j = 0; j < nEdgeStates[i]; j++)
-			edgePot[i][j] = exp(edgePot[i][j]);
+	Update_Pot_Finalize();
 
 	UNPROTECT(1);
 }
@@ -197,15 +189,48 @@ void CRF::Update_Pot()
 		UNPROTECT(1);
 	}
 
-	for (int i = 0; i < nNodes * maxState; i++)
-		nodePot[i] = exp(nodePot[i]);
-	for (int i = 0; i < nEdges; i++)
-		for (int j = 0; j < nEdgeStates[i]; j++)
-			edgePot[i][j] = exp(edgePot[i][j]);
+	Update_Pot_Finalize();
 
 	UNPROTECT(3);
 }
 
+void CRF::Update_Pot_Finalize()
+{
+  double maxPot;
+  for (int i = 0; i < nNodes; i++)
+  {
+    maxPot = 0;
+    for (int j = 0; j < nStates[i]; j++)
+      maxPot = max(maxPot, NodePot(i, j));
+    for (int j = 0; j < nStates[i]; j++)
+      NodePot(i, j) -= maxPot;
+  }
+  
+  int n1, n2;
+  for (int i = 0; i < nEdges; i++)
+  {
+    n1 = EdgesBegin(i);
+    n2 = EdgesEnd(i);
+    maxPot = 0;
+    for (int j = 0; j < nStates[n2]; j++)
+    {
+      for (int k = 0; k < nStates[n1]; k++)
+        maxPot = max(maxPot, EdgePot(i, k, j));
+    }
+    for (int j = 0; j < nStates[n2]; j++)
+    {
+      for (int k = 0; k < nStates[n1]; k++)
+        EdgePot(i, k, j) -= maxPot;
+    }
+  }
+  
+  for (int i = 0; i < nNodes * maxState; i++)
+    nodePot[i] = max(exp(nodePot[i]), 1e-8);
+  for (int i = 0; i < nEdges; i++)
+    for (int j = 0; j < nEdgeStates[i]; j++)
+      edgePot[i][j] = max(exp(edgePot[i][j]), 1e-8);
+}
+
 SEXP MRF_Stat(SEXP _crf, SEXP _instances)
 {
 	CRF crf(_crf);
